--	Generate integrity checks for all application tables.
--	SYSCAT.REFERENCES contains the dependency from child to parent. 
--	SYSCAT.KEYCOLUSE contains one row per foreign key column and one row per primary key column for each referential constraint.
--	We query the KEYCOLUSE view twice because there is no guarantee or restriction that the child and parent column names will always match, 
--  (though best practice is that the column names do match, since they are the same attribute).

--  Note: this SQL compares the values for each single PK column against the corresponding FK column in the child.
--  That is, if the PK ontains 2 columns, it will generate 2 statements.
--  This is not fool-proof for composite primary keys, since it is the set of keys - the combination - that should be compared.
--  This could be rectified using compound or recursive SQL, however the modelling fashion these days seems to be for single column, 
--  surrogate or identity primary keys. In any event, it will not produce false positives and is an adequate approximation for most purposes.

--  Ignore explain tables.
--  Also filter out null values in nullable foreign key columns (I thought this had been banned under the Geneva Convention).
--  Alex, June 2012.
									
select	'select distinct child.', childcol, 							
		' from ', rtrim(rtrim(childschema)||'.'||childtable)||' child',				
		' where ', rtrim(childcol)||' is not null',
		' EXCEPT',																
        ' select distinct parent.', rtrim(parentcol)||' ',		 							
		' from ', rtrim(rtrim(parentschema)||'.'||parenttable)||' parent;'				
from (																	
		select 	substr(r.tabschema,1,30)    as childschema,						
				substr(r.tabname,1,60)      as childtable,						
				substr(k1.colname,1,60)	    as childcol,						
				substr(r.reftabschema,1,30) as parentschema,					
				substr(r.reftabname,1,60)   as parenttable,						
				substr(k2.colname,1,60)	    as parentcol 		
		from 	syscat.references r										
    					inner join											
				syscat.keycoluse k1										
							on  r.constname = k1.constname						
							and r.tabschema = k1.tabschema						
							and r.tabname = k1.tabname							
						inner join											
				syscat.keycoluse k2										
							on  r.refkeyname = k2.constname						
							and r.reftabschema = k2.tabschema					
							and r.reftabname = k2.tabname						
							and k1.colseq = k2.colseq						
		where	r.tabschema not like 'SYS%'               				
		and  	r.tabschema not like 'SQL%'               				
		and  	r.tabschema not like 'NULL%'              				
		and		r.tabname   not like 'ADVISE@_%' escape '@'
		and		r.tabname   not like 'EXPLAIN@_%' escape '@'
		) zzzz	
;									
